from openpyxl.styles.alignment import Alignment
from openpyxl.styles.borders import Border
from openpyxl.styles.cell_style import (
    CellStyle,
    StyleArray,
)
from openpyxl.styles.fills import PatternFill
from openpyxl.styles.fonts import Font
from openpyxl.styles.protection import Protection
from openpyxl.workbook.workbook import Workbook
from typing import (
    Any,
    Iterator,
    List,
    Optional,
    Tuple,
    Union,
)

class NamedStyle:
    def __init__(
        self,
        name: str = ...,
        font: Font = ...,
        fill: PatternFill = ...,
        border: Border = ...,
        alignment: Alignment = ...,
        number_format: Optional[str] = ...,
        protection: Protection = ...,
        builtinId: Optional[Union[str, int]] = ...,
        hidden: Optional[Union[str, bool]] = ...,
        xfId: Optional[Union[str, int]] = ...,
    ) -> None: ...
    def __iter__(self) -> Iterator[Tuple[str, str]]: ...
    def __setattr__(self, attr: str, value: Any) -> None: ...
    def _recalculate(self) -> None: ...
    def _set_index(self, idx: int) -> None: ...
    def as_name(self) -> _NamedCellStyle: ...
    def as_tuple(self) -> StyleArray: ...
    def as_xf(self) -> CellStyle: ...
    def bind(self, wb: Workbook) -> None: ...
    @property
    def xfId(self) -> int: ...

class NamedStyleList:
    def __getitem__(self, key: Union[str, int]) -> NamedStyle: ...
    def append(self, style: Union[NamedStyle, int]) -> None: ...
    @property
    def names(self) -> List[str]: ...

class _NamedCellStyle:
    def __init__(
        self,
        name: Optional[str] = ...,
        xfId: Optional[Union[str, int]] = ...,
        builtinId: Optional[Union[str, int]] = ...,
        iLevel: None = ...,
        hidden: Optional[Union[str, bool]] = ...,
        customBuiltin: Optional[str] = ...,
        extLst: None = ...,
    ) -> None: ...

class _NamedCellStyleList:
    def __init__(
        self,
        count: Optional[str] = ...,
        cellStyle: Union[List[_NamedCellStyle], Tuple] = ...,
    ) -> None: ...
    @property
    def count(self) -> int: ...
    @property
    def names(self) -> NamedStyleList: ...



from io import (
    BufferedReader,
    BytesIO,
)
from openpyxl.packaging.manifest import (
    Manifest,
    Override,
)
from openpyxl.packaging.relationship import Relationship
from openpyxl.packaging.workbook import ChildSheet
from openpyxl.workbook.workbook import Workbook
from typing import Union
from zipfile import ZipFile

def _find_workbook_part(package: Manifest) -> Override: ...
def _validate_archive(filename: Union[str, BufferedReader, BytesIO]) -> ZipFile: ...
def load_workbook(
    filename: Union[str, BufferedReader, BytesIO],
    read_only: bool = ...,
    keep_vba: bool = ...,
    data_only: bool = ...,
    keep_links: bool = ...,
) -> Workbook: ...

class ExcelReader:
    def __init__(
        self,
        fn: Union[str, BufferedReader, BytesIO],
        read_only: bool = ...,
        keep_vba: bool = ...,
        data_only: bool = ...,
        keep_links: bool = ...,
    ) -> None: ...
    def read(self) -> None: ...
    def read_chartsheet(self, sheet: ChildSheet, rel: Relationship) -> None: ...
    def read_manifest(self) -> None: ...
    def read_properties(self) -> None: ...
    def read_strings(self) -> None: ...
    def read_theme(self) -> None: ...
    def read_workbook(self) -> None: ...
    def read_worksheets(self) -> None: ...



from openpyxl.chartsheet.chartsheet import Chartsheet
from openpyxl.styles.named_styles import NamedStyle
from openpyxl.workbook.defined_name import DefinedName
from openpyxl.worksheet._read_only import ReadOnlyWorksheet
from openpyxl.worksheet._write_only import WriteOnlyWorksheet
from openpyxl.worksheet.worksheet import Worksheet
from typing import (
    List,
    Optional,
    Union,
)

class Workbook:
    def __contains__(self, key: str) -> bool: ...
    def __delitem__(self, key: str) -> None: ...
    def __getitem__(
        self, key: str
    ) -> Union[Chartsheet, Worksheet, ReadOnlyWorksheet]: ...
    def __init__(self, write_only: bool = ..., iso_dates: bool = ...) -> None: ...
    def _add_sheet(
        self,
        sheet: Union[Chartsheet, str, WriteOnlyWorksheet, Worksheet],
        index: None = ...,
    ) -> None: ...
    def _duplicate_name(self, name: str) -> Optional[bool]: ...
    def _setup_styles(self) -> None: ...
    def add_named_range(self, named_range: DefinedName) -> None: ...
    def add_named_style(self, style: NamedStyle) -> None: ...
    @property
    def chartsheets(self) -> List[Chartsheet]: ...
    def close(self) -> None: ...
    def copy_worksheet(
        self, from_worksheet: Union[WriteOnlyWorksheet, Worksheet]
    ) -> Worksheet: ...
    def create_chartsheet(self, title: None = ..., index: None = ...) -> Chartsheet: ...
    def create_named_range(
        self,
        name: str,
        worksheet: Optional[Worksheet] = ...,
        value: Optional[str] = ...,
        scope: Optional[int] = ...,
    ) -> None: ...
    def create_sheet(
        self, title: Optional[Union[str, int]] = ..., index: None = ...
    ) -> Union[WriteOnlyWorksheet, Worksheet]: ...
    @property
    def data_only(self) -> bool: ...
    @property
    def excel_base_date(self) -> float: ...
    def get_named_ranges(self) -> List[DefinedName]: ...
    def index(self, worksheet: Worksheet) -> int: ...
    @property
    def mime_type(self) -> str: ...
    def move_sheet(self, sheet: str, offset: int = ...) -> None: ...
    @property
    def named_styles(self) -> List[str]: ...
    @property
    def read_only(self) -> bool: ...
    def remove(self, worksheet: Union[Chartsheet, Worksheet]) -> None: ...
    def save(self, filename: str) -> None: ...
    @property
    def sheetnames(self) -> List[str]: ...
    @property
    def worksheets(self) -> Union[List[Worksheet], List[ReadOnlyWorksheet]]: ...
    @property
    def write_only(self) -> bool: ...



from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.drawing.colors import SystemColor
from typing import (
    Iterator,
    List,
    Optional,
    Tuple,
    Union,
)

# Default Color Index as per 18.8.27 of ECMA Part 4
COLOR_INDEX = (
    "00000000",
    "00FFFFFF",
    "00FF0000",
    "0000FF00",
    "000000FF",  # 0-4
    "00FFFF00",
    "00FF00FF",
    "0000FFFF",
    "00000000",
    "00FFFFFF",  # 5-9
    "00FF0000",
    "0000FF00",
    "000000FF",
    "00FFFF00",
    "00FF00FF",  # 10-14
    "0000FFFF",
    "00800000",
    "00008000",
    "00000080",
    "00808000",  # 15-19
    "00800080",
    "00008080",
    "00C0C0C0",
    "00808080",
    "009999FF",  # 20-24
    "00993366",
    "00FFFFCC",
    "00CCFFFF",
    "00660066",
    "00FF8080",  # 25-29
    "000066CC",
    "00CCCCFF",
    "00000080",
    "00FF00FF",
    "00FFFF00",  # 30-34
    "0000FFFF",
    "00800080",
    "00800000",
    "00008080",
    "000000FF",  # 35-39
    "0000CCFF",
    "00CCFFFF",
    "00CCFFCC",
    "00FFFF99",
    "0099CCFF",  # 40-44
    "00FF99CC",
    "00CC99FF",
    "00FFCC99",
    "003366FF",
    "0033CCCC",  # 45-49
    "0099CC00",
    "00FFCC00",
    "00FF9900",
    "00FF6600",
    "00666699",  # 50-54
    "00969696",
    "00003366",
    "00339966",
    "00003300",
    "00333300",  # 55-59
    "00993300",
    "00993366",
    "00333399",
    "00333333",  # 60-63
)

BLACK = COLOR_INDEX[0]
WHITE = COLOR_INDEX[1]
BLUE = COLOR_INDEX[4]

aRGB_REGEX = re.compile("^([A-Fa-f0-9]{8}|[A-Fa-f0-9]{6})$")

class Color:
    def __add__(self, other: Union[Color, int]) -> Color: ...
    def __init__(
        self,
        rgb: str = ...,
        indexed: Optional[Union[str, int]] = ...,
        auto: Optional[Union[str, bool, int]] = ...,
        theme: Optional[Union[str, int]] = ...,
        tint: Union[float, str] = ...,
        index: None = ...,
        type: str = ...,
    ) -> None: ...
    def __iter__(self) -> Iterator[Tuple[str, str]]: ...
    @property
    def index(self) -> int: ...

class ColorDescriptor:
    def __set__(
        self, instance: Serialisable, value: Optional[Union[str, Color]]
    ) -> None: ...

class ColorList:
    def __bool__(self) -> bool: ...
    def __init__(
        self,
        indexedColors: Union[
            Tuple[str, str],
            List[str],
            Tuple,
            List[RgbColor],
            Tuple[
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
                str,
            ],
        ] = ...,
        mruColors: Union[List[Color], Tuple] = ...,
    ) -> None: ...
    @property
    def index(self) -> List[str]: ...

class RGB:
    def __set__(
        self,
        instance: Union[Color, SystemColor, RgbColor],
        value: Optional[Union[str, int]],
    ) -> None: ...

class RgbColor:
    def __init__(self, rgb: Optional[str] = ...) -> None: ...
